@@ -1,15 +1,7 @@
-/* $Id: dom.c 694 2007-11-12 09:06:59Z pajas $ */
-#include <libxml/tree.h>
-#include <libxml/encoding.h>
-#include <libxml/xmlerror.h>
-#include <libxml/xmlmemory.h>
-#include <libxml/parser.h>
-#include <libxml/parserInternals.h>
-#include <libxml/xmlIO.h>
-#include <libxml/xpath.h>
-#include <libxml/xpathInternals.h>
-#include <libxml/globals.h>
-#include <stdio.h>
+/* $Id: dom.c 747 2008-11-03 14:29:15Z pajas $ */
+
+#include "dom.h"
+
 
 /* #define warn(string) fprintf(stderr, string) */
 
@@ -661,8 +653,7 @@ domAppendChild( xmlNodePtr self,
 
     if ( !(domTestHierarchy(self, newChild)
            && domTestDocument(self, newChild))){
-        xs_warn("HIERARCHY_REQUEST_ERR\n"); 
-        xmlGenericError(xmlGenericErrorContext,"HIERARCHY_REQUEST_ERR\n");
+        croak("appendChild: HIERARCHY_REQUEST_ERR\n"); 
         return NULL;
     }
 
@@ -755,8 +746,7 @@ domReplaceChild( xmlNodePtr self, xmlNodePtr new, xmlNodePtr old ) {
 
     if ( !(domTestHierarchy(self, new)
            && domTestDocument(self, new))){
-        xs_warn("HIERARCHY_REQUEST_ERR\n"); 
-        xmlGenericError(xmlGenericErrorContext,"HIERARCHY_REQUEST_ERR\n");
+        croak("replaceChild: HIERARCHY_REQUEST_ERR\n");
         return NULL;
     }
     
@@ -815,7 +805,7 @@ domInsertBefore( xmlNodePtr self,
    
     if ( !(domTestHierarchy( self, newChild )
            && domTestDocument( self, newChild ))) {
-        xmlGenericError(xmlGenericErrorContext,"HIERARCHY_REQUEST_ERR\n");
+        croak("insertBefore/insertAfter: HIERARCHY_REQUEST_ERR\n");
         return NULL;
     }
 
@@ -880,7 +870,7 @@ domReplaceNode( xmlNodePtr oldNode, xmlNodePtr newNode ) {
          * wrong node type
          * new node is parent of itself
          */
-        xmlGenericError(xmlGenericErrorContext,"HIERARCHY_REQUEST_ERR\n");
+        croak("replaceNode: HIERARCHY_REQUEST_ERR\n");
         return NULL;
     }
         
