@@ -1,5 +1,5 @@
 /* dom.h
- * $Id: dom.h 647 2006-12-19 22:30:54Z pajas $
+ * $Id: dom.h 747 2008-11-03 14:29:15Z pajas $
  * Author: Christian Glahn (2001)
  * 
  * This header file provides some definitions for wrapper functions.
@@ -20,8 +20,30 @@
 #ifndef __LIBXML_DOM_H__
 #define __LIBXML_DOM_H__
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+#include "EXTERN.h"
+#include "perl.h"
+#include "XSUB.h"
+#include "ppport.h"
+
 #include <libxml/tree.h>
 #include <libxml/xpath.h>
+#include <libxml/encoding.h>
+#include <libxml/xmlerror.h>
+#include <libxml/xmlmemory.h>
+#include <libxml/parser.h>
+#include <libxml/parserInternals.h>
+#include <libxml/xmlIO.h>
+#include <libxml/xpathInternals.h>
+#include <libxml/globals.h>
+#include <stdio.h>
+
+#ifdef __cplusplus
+}
+#endif
 
 /**
  * part 0:
